ad_page_contract {
}

set user_id [ad_maybe_redirect_for_registration]
set package_id [ad_conn package_id]
permission::require_permission -object_id $package_id -privilege mailing_list_admin

set context_bar [list [list "." Administration] "Mass Subscribe to Mailing List"]

set lists [db_list_of_lists get_mailing_lists {}]


ad_form -name list_mass_subscribe_form -action list-subscribe -form {
    {list_id:text(select) {label "Mailing List"} {options $lists}}
    {xx:text(inform) {label " "} {value "Enter emails and 2-character country code as a comma-seperated list (csv)"}}
    {emails_countries:text(textarea) {label "Emails and Countries"} {html {rows 30 cols 80}}}
    {confirmation_p:text(checkbox),optional {label "Send confirmation?"} {options {{"Yes" t}}}}
} -validate {
    {emails_countries {
	[empty_string_p [set invalid_list [mailing_list::util::check_valid_country_codes -emails_and_countries $emails_countries]]]}
	"Invalid country code for emails: [join $invalid_list , ]"}
    {emails_countries {
	[empty_string_p [set invalid_list [mailing_list::util::check_valid_emails -emails_and_countries $emails_countries]]]}
	"Invalid emails: [join $invalid_list , ]"}
} -on_submit {
    permission::require_permission -object_id $list_id -privilege read

    if {[exists_and_not_null confirmation_p]} {
	set confirmed_p f
    } else {
	set confirmed_p t
    }
    set default_first_names "Unknown"
    set default_last_name "User"
    set default_question "q"
    set default_answer "a"

    db_1row get_list_mail_data {}

    # get all mappings country_code -> country_id so that we can map users
    # to country_ids a lot easier
    db_foreach get_country_codes {} {
	set country_id($country_code) $category_id
    }

    # generate tmp-file for csv-processing
    set tmp_file [ns_tmpnam]
    set fd [open $tmp_file w]
    puts $fd $emails_countries
    close $fd
    set fd [open $tmp_file r]

    set unsubscribe_link "[ad_url][ad_conn package_url]"

    db_transaction {
	# process every line of the user/country list
	while {[ns_getcsv $fd line] > 0} {

	    set user_id ""
	    # grap the email and country of the current line
	    util_unlist $line email country

	    # check if user already exists
	    set user_id [cc_email_user $email]

	    if {[empty_string_p $user_id]} {
		# user doesn't exist, so create him
		set user_password [ad_generate_random_string]
		set user_id [ad_user_new $email $default_first_names $default_last_name $user_password $default_question $default_answer "" $confirmed_p]
	    } else {
		set user_password ""
	    }

	    set confirmation_link "[ad_url][ad_conn package_url]confirm?[export_url_vars list_id user_id]"

	    # try to map user to the given country - this might fail if user
	    # is already mapped
	    catch {[category::map_object -remove_old -object_id $user_id $country_id($country)]}

	    # subscribe user. check if user already has a map entry
	    if {[mailing_list::add_user -list_id $list_id -user_id $user_id -confirmed_p $confirmed_p]} {
		# user is newly added to this list
		# now send confirmation message to user if requested
		if {$confirmed_p == "f"} {
		    mailing_list::util::send_mail -user_id $user_id -from_email $sender_email -subject $confirm_subject -body $confirm_body -mime_type $confirm_mime_type -charset $mime_charset -link $confirmation_link -new_password $user_password
		} else {
		    # send welcome message if no confirmation needed
		    mailing_list::util::send_mail -user_id $user_id -from_email $sender_email -subject $welcome_subject -body $welcome_body -mime_type $welcome_mime_type -charset $mime_charset -link "$unsubscribe_link?[export_url_vars user_id]" -new_password $user_password
		}
	    }
	}
    }

    # remove tmp-file for csv processing
    close $fd
    ns_unlink $tmp_file
} -after_submit {
    ad_returnredirect "."
    ad_script_abort
}

ad_return_template
